//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGEffect.h
// Description : Classe permettant de stocker les informations necessaires
//               a un effet multi-passes Cg.
//===========================================================================


#ifndef __CGEFFECT_H
#define __CGEFFECT_H

#include "./CGPass.h"
#include "./CGPixelShader.h"
#include "./CGVertexShader.h"


//-------------------------------------------------------------------------
class CCGEffect
{

public :

   // TODO : avoir un nombre de passes quelconque serait bien
   #define MAX_PASS	10
   
   //----------------------------------------------------------------
   CCGEffect( void ) ;
   virtual ~CCGEffect( void ) ;
   // TODO : un constructeur de copie serait certainement bien utile
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // ACCESSEURS
   CCGPass * getPass( unsigned int num ) const ;

   CCGPass *    getCurrentPass(    void ) const { return m_pAllPass[m_uiCurrentPass] ; }
   unsigned int getCurrentPassNum( void ) const { return m_uiCurrentPass             ; }

   unsigned int getNbPass( void ) const { return m_uiNbPass ; }
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // INTERFACE UTILISATION
   CCGPass * getFirstPass( void ) ;
   CCGPass * getNextPass(  void ) ;
   
   CCGPass * getLastPass(     void ) ;
   CCGPass * getPreviousPass( void ) ;
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // MODIFIEURS
   bool pushPass( CCGVertexShader * vs                                    ,
                  CCGPixelShader  * ps                                    ,
	          bool              zenable       = true                  ,
                  bool              zwrite        = false                 ,
	          bool              alpha         = false                 ,
	          bool              blend         = true                  ,
                  GLenum            alphaFunc     = GL_LEQUAL             ,
	          GLclampf          alphaRef      = 0.0f                  ,
                  GLenum            blendSrcFunc  = GL_SRC_ALPHA          ,
                  GLenum            blendDestFunc = GL_ONE_MINUS_SRC_ALPHA ) ;
   void popPass( void ) ;
   //----------------------------------------------------------------

protected :

   CCGPass * m_pAllPass[MAX_PASS] ;

   unsigned int m_uiNbPass      ;
   unsigned int m_uiCurrentPass ;

} ;
//-------------------------------------------------------------------------

#endif //__CGEFFECT_H
