//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGPass.cpp
// Description : Classe permettant de stocker les informations necessaires
//               a une passe pour une effet Cg.
//===========================================================================


#include <stdio.h>
#include "./CGPass.h"


//-------------------------------------------------------------------------
// CONSTRUCTEURS - DESTRUCTEUR --------------------------------------------
//-------------------------------------------------------------------------

CCGPass::CCGPass( void )
{
   m_pVS = NULL ;
   m_pPS = NULL ;

   m_bZEnable = true  ;
   m_bZWrite  = false ;
   m_bAlpha   = false ;
   m_bBlend   = true  ;

   m_eAlphaFunc     = GL_LEQUAL              ;
   m_eBlendSrcFunc  = GL_SRC_ALPHA           ;
   m_eBlendDestFunc = GL_ONE_MINUS_SRC_ALPHA ;
   
   m_fAlphaRef = 0.0f ;
}

//-------------------------------------------------------------------------

CCGPass::CCGPass( CCGVertexShader * vs           ,
                  CCGPixelShader  * ps           ,
                  bool              zenable      ,
                  bool              zwrite       ,
                  bool              alpha        ,
		  bool              blend        ,
                  GLenum            alphaFunc    ,
	          GLclampf          alphaRef     ,
                  GLenum            blendSrcFunc ,
                  GLenum            blendDestFunc )
{
   m_pVS = vs ;
   m_pPS = ps ;

   m_bZEnable = zenable ;
   m_bZWrite  = zwrite  ;
   m_bAlpha   = alpha   ;
   m_bBlend   = blend   ;

   m_eAlphaFunc     = alphaFunc     ;
   m_eBlendSrcFunc  = blendSrcFunc  ;
   m_eBlendDestFunc = blendDestFunc ;
   
   m_fAlphaRef      = alphaRef      ;
}

//-------------------------------------------------------------------------

CCGPass::~CCGPass( void )
{
}





//-------------------------------------------------------------------------
// INTERFACE UTILISATION --------------------------------------------------
//-------------------------------------------------------------------------

void CCGPass::enable( void )
{
   // test de profondeur
   if( m_bZEnable ) { glEnable(  GL_DEPTH_TEST ) ; }
   else             { glDisable( GL_DEPTH_TEST ) ; }

   // ecriture dans le buffer de profondeur
   if(  m_bZWrite  ) { glDepthMask( GL_TRUE  ) ; }
   else              { glDepthMask( GL_FALSE ) ; }

   // test d'alpha
   if( m_bAlpha )
   {
      glEnable( GL_ALPHA_TEST ) ;
      glAlphaFunc( m_eAlphaFunc, m_fAlphaRef ) ;
   }
   else { glDisable( GL_ALPHA_TEST ) ; }
   
   // mise en place des operations de blending
   if( m_bBlend )
   {
      glEnable( GL_BLEND ) ;
      glBlendFunc( m_eBlendSrcFunc, m_eBlendDestFunc ) ;
   }
   else { glDisable( GL_BLEND ) ; }

   // activation des shaders associes
   if( m_pVS != NULL ) { m_pVS->enable() ; }
   if( m_pPS != NULL ) { m_pPS->enable() ; }
}

//-------------------------------------------------------------------------

void CCGPass::disable( void )
{
   // desactivation des shaders associes
   if( m_pVS != NULL ) { m_pVS->disable() ; }
   if( m_pPS != NULL ) { m_pPS->disable() ; }
}





//-------------------------------------------------------------------------
// MODIFIEURS -------------------------------------------------------------
//-------------------------------------------------------------------------

void CCGPass::setVertexShader( CCGVertexShader * vs )
{
   m_pVS = vs ;
}

//-------------------------------------------------------------------------

void CCGPass::setPixelShader( CCGPixelShader * ps )
{
   m_pPS = ps ;
}

//-------------------------------------------------------------------------
   
void CCGPass::setMeltingParams( bool     zenable      ,
                                bool     zwrite       ,
                                bool     alpha        ,
				bool     blend        ,
                                GLenum   alphaFunc    ,
				GLclampf alphaRef     ,
                                GLenum   blendSrcFunc ,
                                GLenum   blendDestFunc )
{
   m_bZEnable = zenable ;
   m_bZWrite  = zwrite  ;
   m_bAlpha   = alpha   ;
   m_bBlend   = blend   ;

   m_eAlphaFunc     = alphaFunc     ;
   m_eBlendSrcFunc  = blendSrcFunc  ;
   m_eBlendDestFunc = blendDestFunc ;
   
   m_fAlphaRef      = alphaRef      ;
}

//-------------------------------------------------------------------------

void CCGPass::setZEnable( bool truth )
{
   m_bZEnable = truth ;
}

//-------------------------------------------------------------------------

void CCGPass::setZWrite( bool truth )
{
   m_bZWrite = truth ;
}

//-------------------------------------------------------------------------

void CCGPass::setAlpha( bool truth )
{
   m_bAlpha = truth ;
}

//-------------------------------------------------------------------------

void CCGPass::setBlend( bool truth )
{
   m_bBlend = truth ;
}

//-------------------------------------------------------------------------

void CCGPass::setAlphaRef( GLclampf ref )
{
   m_fAlphaRef = ref ;
}

//-------------------------------------------------------------------------

void CCGPass::setAlphaFunc( GLenum func )
{
   m_eAlphaFunc = func ;
}

//-------------------------------------------------------------------------

void CCGPass::setBlendSrcFunc(  GLenum func )
{
   m_eBlendSrcFunc = func ;
}

//-------------------------------------------------------------------------

void CCGPass::setBlendDestFunc( GLenum func )
{
   m_eBlendDestFunc = func ;
}

//-------------------------------------------------------------------------
