//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGShaderManager.cpp
// Description : Classe permettant de gerer des Shaders ecrit en CG.
//===========================================================================


#ifndef __CGSHADERMANAGER_H
#define __CGSHADERMANAGER_H

#include "./CGVertexShader.h"
#include "./CGPixelShader.h"


//-------------------------------------------------------------------------
class CCGShaderManager
{

public :

   //----------------------------------------------------------------
   // Type de shader
   typedef  unsigned int            EShaderType ;
   #define  CGSM_VERTEX_SHADER      0x1
   #define  CGSM_PIXEL_SHADER       0x2
   //----------------------------------------------------------------



   //----------------------------------------------------------------
   // CONSTRUCTEURS - DESTRUCTEUR
   CCGShaderManager( void ) ;
   virtual ~CCGShaderManager( void ) ;
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // CONSTRUCTEURS DE SHADERS
   CCGVertexShader * createVertexShaderFromSourceFile( const char * filename, const char * entry = 0 ) ;
   CCGPixelShader  * createPixelShaderFromSourceFile(  const char * filename, const char * entry = 0 ) ;

   bool manageExistingVertexShader( CCGVertexShader * shader, const char * filename ) ;
   bool manageExistingPixelShader(  CCGPixelShader  * shader, const char * filename ) ;
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // ACCESSEURS
   CCGVertexShader * getVertexShader( const char * filename ) const ;
   CCGPixelShader  * getPixelShader(  const char * filename ) const ;
   
   CCGShader * getShader( EShaderType type, const char * filename ) const ;

   const char * getLastListing( void ) const { return cgGetLastListing( m_oGlobalContext ) ; }

   CGcontext getGlobalContext( void ) const { return m_oGlobalContext ; }
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // MODIFIEURS
   void recompileAll( EShaderType type = CGSM_VERTEX_SHADER | CGSM_PIXEL_SHADER ) ;

   void recompileVertexShader( const char * filename ) ;
   void recompilePixelShader(  const char * filename ) ;

   void recompileShader( EShaderType type, const char * filename ) ;
   //----------------------------------------------------------------

protected :

   // TODO : ameliorer la gestion des shaders puisque le manager est la
   // pour ca et puique pour l'instant c'est pas top !! Un arbre binaire est
   // biensur une bonne idee :) mais il y a peut etre mieux ...
   struct __internal_node
   {
      CCGShader * m_pShader ;
      char      * m_sName   ;
   } ;

   #define NB_MAX_SHADERS  10
   
   __internal_node m_pPixelShaders[NB_MAX_SHADERS]  ;
   __internal_node m_pVertexShaders[NB_MAX_SHADERS] ;

   int m_iNbPixelShaders  ;
   int m_iNbVertexShaders ;

   // TODO : gerer la hierarchie entre les contextes et les shaders
   // serait une bonne chose ...
   CGcontext m_oGlobalContext ;

} ;
//-------------------------------------------------------------------------

#endif //__CGSHADERMANAGER_H
