//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGToonimVS.h
// Description : Classe pour le Vertex Shader de Toon Shading.
//               
//===========================================================================


#ifndef __CGTOONIMVS_H
#define __CGTOONIMVS_H

#include <QGLViewer/qglviewer.h>
#include "./CGVertexShader.h"


//-------------------------------------------------------------------------
class CCGToonimVS : public CCGVertexShader
{

public :

   //----------------------------------------------------------------
   // CONSTRUCTEURS - DESTRUCTEURS
   CCGToonimVS( CGcontext context, const char * file, const char * entry )
      : CCGVertexShader( context, file, entry ) {}
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // INTERFACE UTILISATION
   void enable ( void ) ;
   void disable( void ) ;
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // PARAMETRAGE DU SHADER
   void setDeplParamName               ( const char * name ) ;
   void setPuissance1ParamName         ( const char * name ) ;
   void setPuissance2ParamName         ( const char * name ) ;
   void setLight1PosParamName          ( const char * name ) ;
   void setLight1DirParamName          ( const char * name ) ;
   void setLight2PosParamName          ( const char * name ) ;
   void setLight2DirParamName          ( const char * name ) ;
   void setModelViewProjMatrixParamName( const char * name ) ;
   void setEyePositionParamName        ( const char * name ) ;
   

   // parametres uniformes constants
   void setCU_Depl      ( const float val ) ;
   void setCU_Puissance1( const float val ) ;
   void setCU_Puissance2( const float val ) ;
   void setCU_Light1Pos ( float V[3] ) ;
   void setCU_Light1Dir ( float V[3] ) ;
   void setCU_Light2Pos ( float V[3] ) ;
   void setCU_Light2Dir ( float V[3] ) ;

   // parametres uniformes
   void setU_EyePosition( const qglviewer::Camera * cam ) ;
   
   //----------------------------------------------------------------

private :

   CGparameter m_cgpModelViewProjMatrix ;


   CGparameter m_cgpDepl        ;
   CGparameter m_cgpPuissance1  ;
   CGparameter m_cgpPuissance2  ;
   CGparameter m_cgpLight1Pos   ;
   CGparameter m_cgpLight1Dir   ;
   CGparameter m_cgpLight2Pos   ;
   CGparameter m_cgpLight2Dir   ;
   CGparameter m_cgpEyePosition ;
   
   float m_pDepl         ;
   float m_pPuissance1   ;
   float m_pPuissance2   ;
   float m_pLight1Pos[3] ;
   float m_pLight1Dir[3] ;
   float m_pLight2Pos[3] ;
   float m_pLight2Dir[3] ;
   const qglviewer::Camera * m_pEyePosition ;

} ;
//-------------------------------------------------------------------------

#endif //__CGTOONIMVSVS_H
