//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : GLLight.cpp
// Description : Classe representant une source de lumière
//===========================================================================


#include <stdio.h>
#include <math.h>
#include "./GLLight.h"

//-------------------------------------------------------------------------
// CONSTRUCTEURS-DESTRUCTEURS ---------------------------------------------
//-------------------------------------------------------------------------

CGLLight::CGLLight( const float Pos[3], const float Dir[3],
                    const float Coul[3], const float Puiss,
                    const float Angle, const float Echelle )
{
   Puissance = Puiss;
   Angle_Cone = Angle;
   Taille = Echelle;
   // on normalize la direction
   float Norm = sqrt( Dir[0]*Dir[0] + Dir[1]*Dir[1] + Dir[2]*Dir[2] );
   for ( int i = 0; i < 3 ; i++ ) {
      Position[i] = Pos[i];
      Direction[i] = Dir[i]/Norm;
      Couleur[i] = Coul[i];
   }
}
   
   
   
CGLLight::CGLLight( void )
{
   Puissance = 0.0;
   Angle_Cone = 0.0;
   Taille = 0.0;
   for ( int i=0; i < 3 ; i++ ) {
      Position[i] = 0.0;
      Direction[i] = 0.0;
      Couleur[i] = 0.0;
   }
}


//-------------------------------------------------------------------------
// INTERFACE UTILISATION --------------------------------------------------
//-------------------------------------------------------------------------

void CGLLight::draw( void )
{
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   glTranslatef( Position[0], Position[1], Position[2] );
   // Calcul du produit vectoriel entre la direction actuelle (0,0,1)
   // et la direction, c'est autour de cet axe que l'on doit faire la
   // rotation
   float CrossP[3] = { -Direction[1], Direction[0], 0 };
   // l'angle de la rotation est alors l'angle du cosinus entre
   // la direction actuelle et la direction.
   float angle = acos( Direction[2] );
   glRotatef(angle*180.0/M_PI,CrossP[0],CrossP[1], CrossP[2]);
   drawPointe();
   drawCylindre();
   drawDisque();
   glPopMatrix();
   
}

//-------------------------------------------------------------------------

// A modifier
void CGLLight::switch_on( void )
{
   Puissance  *= 1000.0f;
   Couleur[0] *= 2;
   Couleur[1] *= 2;
   Couleur[2] *= 2;
}

//-------------------------------------------------------------------------

// A modifier
void CGLLight::switch_off( void )
{
   Puissance  /= 1000.0f;
   Couleur[0] /= 2;
   Couleur[1] /= 2;
   Couleur[2] /= 2;
}

//-------------------------------------------------------------------------
// PARAMETRAGE ------------------------------------------------------------
//-------------------------------------------------------------------------

void CGLLight::Set_Position( const float Pos[3] )
{
   Position[0] = Pos[0];
   Position[1] = Pos[1];
   Position[2] = Pos[2];
}
   
//-------------------------------------------------------------------------
   
void CGLLight::Set_Direction( const float Dir[3] )
{
   // on normalize la direction
   float Norm = sqrt( Dir[0]*Dir[0] + Dir[1]*Dir[1] + Dir[2]*Dir[2] );
   Direction[0] = Dir[0]/Norm;
   Direction[1] = Dir[1]/Norm;
   Direction[2] = Dir[2]/Norm;
}


//-------------------------------------------------------------------------
   
void CGLLight::Set_Couleur( const float Coul[3] )
{
   Couleur[0] = Coul[0];
   Couleur[1] = Coul[1];
   Couleur[2] = Coul[2];
}

//-------------------------------------------------------------------------

void CGLLight::Set_Taille( const float Echelle )
{
   Taille = Echelle;
}

//-------------------------------------------------------------------------

void CGLLight::Set_Puissance( const float Puiss )
{
   Puissance = Puiss;
}

//-------------------------------------------------------------------------

void CGLLight::Set_Angle_Cone( const float Angle )
{
   Angle_Cone = Angle;
}


//-------------------------------------------------------------------------
// ACCESSEURS -------------------------------------------------------------
//-------------------------------------------------------------------------

void CGLLight::Get_Position( float Pos[3] )
{
   
   Pos[0] = Position[0];
   Pos[1] = Position[1];
   Pos[2] = Position[2];
}

//-------------------------------------------------------------------------

void CGLLight::Get_Direction( float Dir[3] )
{
   Dir[0] = Direction[0];
   Dir[1] = Direction[1];
   Dir[2] = Direction[2];
}

//-------------------------------------------------------------------------

void CGLLight::Get_Couleur( float Coul[3] )
{
   Coul[0] = Couleur[0];
   Coul[1] = Couleur[1];
   Coul[2] = Couleur[2];
}

//-------------------------------------------------------------------------

float CGLLight::Get_Taille( void )
{
   return Taille;
}

//-------------------------------------------------------------------------

float CGLLight::Get_Puissance( void )
{
   return Puissance;
}

//-------------------------------------------------------------------------

float CGLLight::Get_Angle_Cone( void )
{
   return Angle_Cone;
}

//-------------------------------------------------------------------------
// FONCTIONS D'AFFICHAGE --------------------------------------------------
//-------------------------------------------------------------------------

void CGLLight::drawPointe( void )
{
   //glutSolidCone(GLdouble base, GLdouble height, GLint slices, GLint stacks);
   glPushMatrix();
   // On met le sommet du cone en (0,0,0)
   glTranslatef( 0, 0, -Taille );
   glColor3f( Couleur[0], Couleur[1], Couleur[2] );
   glutSolidCone(Taille, Taille, 20, 10);
   glPopMatrix();
}

//-------------------------------------------------------------------------

void CGLLight::drawCylindre( void )
{
   glPushMatrix();
   glTranslatef( 0,0, -2*Taille );
   GLUquadric * quad = gluNewQuadric();
   glColor3f( Couleur[0]*0.75, Couleur[1]*0.75, Couleur[2]*0.75 );
   gluCylinder (quad, Taille/2, Taille/2, Taille, 20, 10);
   gluDeleteQuadric(quad);
   glPopMatrix();
}

void CGLLight::drawDisque( void )
{
   glPushMatrix();
   glTranslatef( 0,0, -2*Taille );
   GLUquadric * quad = gluNewQuadric();
   glColor3f( Couleur[0]*0.6, Couleur[1]*0.6, Couleur[2]*0.6 );
   gluDisk (quad, 0, Taille/2, 20, 10);
   gluDeleteQuadric(quad);
   glPopMatrix();
}

//-------------------------------------------------------------------------

