//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : Toon.cpp
// Description : point d'entree du programme principal
//===========================================================================


#include <stdio.h>
#include <math.h>
#include <qapplication.h>
#include <QGLViewer/qglviewer.h>
#include "./GLWindow.h"
#include "./GLTexture.h"
#include "./CGShaderManager.h"
#include "./CGToonimVS.h"
#include "./CGToonimPS.h"
#include "./CGToondeplVS.h"
#include "./CGToondeplPS.h"
#include "./GLLight.h"


//-------------------------------------------------------------------------
// VARIABLES GLOBALES -----------------------------------------------------
//-------------------------------------------------------------------------

CCGShaderManager * g_pShaderManager = NULL ;
CCGToonimVS  * g_pToonimVS  = NULL ;
CCGToonimPS  * g_pToonimPS  = NULL ;
CCGToondeplVS  * g_pToondeplVS  = NULL ;
CCGToondeplPS  * g_pToondeplPS  = NULL ;

CCGEffect * g_pEffect = NULL ;

//-------------------------------------------------------------------------
// PROGRAMME PRINCIPAL ----------------------------------------------------
//-------------------------------------------------------------------------

int main( int argc, char * argv[] )
{

   // determination des parametres du programme
   const float seuil      = 0.1;
   const float seuilTrait = 0.1;
   const float depl       = 0.01;
   const float pl1        = 0.8;
   const float pl2        = 0.8;
   const float tl1        = 0.5;
   const float tl2        = 0.5;
   const float angle1     = 1.5;
   const float angle2     = 1.5;
   const float LCarac     = 5.0;
   const float posl1[3]   = { 5.0, 0.0, -7.0 };
   const float dirl1[3]   = { 0.0, 0.0, 1.0 };
   const float coul1[3]   = { 1.0, 1.0, 1.0 };
   const float posl2[3]   = { 0.0, 0.0, 5.0 };
   const float dirl2[3]   = { 0.0, 0.0, -1.0 };
   const float coul2[3]   = { 1.0, 1.0, 1.0 };
   float vitesse[3] = { -10.0, 0.0, 0.0 };
   
   QApplication application( argc, argv ) ;

   CGLWindow window ;
   
   // rendre la fenetre visible
   window.show() ;
   

   // utiliser la fenetre comme widget principal
   application.setMainWidget( &window ) ;
  

   // creation des objets 3D ----------------------------
   window.setSceneFile( "57chevy.x3d" ) ;
   // ---------------------------------------------------
   
   // creation des lumieres
   CGLLight * light1 = new CGLLight( posl1, dirl1, coul1, pl1, angle1, tl1 );
   CGLLight * light2 = new CGLLight( posl2, dirl2, coul2, pl2, angle2, tl2 );
   
   window.setLight(light1,0);
   window.setLight(light2,1);
   
   
   // creation du manageur de shaders -------------------
   g_pShaderManager = new CCGShaderManager ;

   g_pToonimVS = new CCGToonimVS( g_pShaderManager->getGlobalContext(), "ToonVP.cg", "ToonVP" ) ;
   g_pShaderManager->manageExistingVertexShader( g_pToonimVS, "ToonVP.cg" ) ;

   if( NULL != g_pShaderManager->getLastListing() )
   {
      printf( "%s\n", g_pShaderManager->getLastListing() ) ;
   }

   g_pToonimPS  = new CCGToonimPS( g_pShaderManager->getGlobalContext(), "ToonFP.cg", "ToonFP" ) ;
   g_pShaderManager->manageExistingPixelShader( g_pToonimPS, "ToonFP.cg" ) ;

   if( NULL != g_pShaderManager->getLastListing() )
   {
      printf( "%s\n", g_pShaderManager->getLastListing() ) ;
   }
   
   g_pToondeplVS = new CCGToondeplVS( g_pShaderManager->getGlobalContext(), "ToonDeplVP.cg", "ToonDeplVP" ) ;
   g_pShaderManager->manageExistingVertexShader( g_pToondeplVS, "ToonDeplVP.cg" ) ;

   if( NULL != g_pShaderManager->getLastListing() )
   {
      printf( "%s\n", g_pShaderManager->getLastListing() ) ;
   }

   g_pToondeplPS  = new CCGToondeplPS( g_pShaderManager->getGlobalContext(), "ToonDeplFP.cg", "ToonDeplFP" ) ;
   g_pShaderManager->manageExistingPixelShader( g_pToondeplPS, "ToonDeplFP.cg" ) ;

   if( NULL != g_pShaderManager->getLastListing() )
   {
      printf( "%s\n", g_pShaderManager->getLastListing() ) ;
   }
   
   // ---------------------------------------------------

   // preparation des effets ----------------------------
   // creation des parametres
   g_pToonimVS->setModelViewProjMatrixParamName( "modelViewProjMatrix" ) ;
   g_pToonimVS->setEyePositionParamName        ( "eyePosition"         ) ;
   g_pToonimVS->setDeplParamName               ( "depl"                ) ;
   g_pToonimVS->setPuissance1ParamName         ( "puissance1"          ) ;
   g_pToonimVS->setPuissance2ParamName         ( "puissance2"          ) ;
   g_pToonimVS->setLight1PosParamName          ( "light1Pos"           ) ;
   g_pToonimVS->setLight1DirParamName          ( "light1Dir"           ) ;
   g_pToonimVS->setLight2PosParamName          ( "light2Pos"           ) ;
   g_pToonimVS->setLight2DirParamName          ( "light2Dir"           ) ;
   
   g_pToonimPS->setEdgeTexParamName   ( "edgeTex"    ) ;
   g_pToonimPS->setDiffuseTexParamName( "diffuseTex" ) ;
   g_pToonimPS->setSeuilParamName     ( "seuil"      ) ;
   g_pToonimPS->setCouleur1ParamName  ( "couleur1"   ) ;
   g_pToonimPS->setCouleur2ParamName  ( "couleur2"   ) ;
   
   
   g_pToondeplVS->setModelViewProjMatrixParamName( "modelViewProjMatrix" ) ;
   g_pToondeplVS->setEyePositionParamName        ( "eyePosition"         ) ;
   g_pToondeplVS->setSeuilTraitParamName         ( "seuilTrait"          ) ;
   g_pToondeplVS->setVitesseParamName            ( "vitesse"             ) ;
   g_pToondeplVS->setLongCaracParamName          ( "longCarac"           ) ;
   
   g_pToondeplPS->setTraitTexParamName( "traitTex") ;

   // mise en place de la texture de bord
   glPixelStorei( GL_UNPACK_ALIGNMENT, 1 ) ;
   CGLTexture tex( "edges.png" ) ;
   tex.createGLTexture() ;
   
   // mise en place de la texture de l'objet
   glPixelStorei( GL_UNPACK_ALIGNMENT, 1 ) ;
   CGLTexture tex2( "diffuse.png" ) ;
   tex2.createGLTexture() ;
   
   // mise en place de la texture des traits de deplacement
   glPixelStorei( GL_UNPACK_ALIGNMENT, 1 ) ;
   CGLTexture tex3( "trait6.png" ) ;
   tex3.createGLTexture() ;
   
   // initialisation des parametres constants du VS de toon shading
   g_pToonimVS->setCU_Depl ( depl ) ;
   g_pToonimVS->setCU_Puissance1( light1->Get_Puissance() ) ;
   g_pToonimVS->setCU_Puissance2( light2->Get_Puissance() ) ;
   float V[3];
   light1->Get_Position( V );
   g_pToonimVS->setCU_Light1Pos( V ) ;
   light1->Get_Direction( V );
   g_pToonimVS->setCU_Light1Dir( V ) ;
   light2->Get_Position( V );
   g_pToonimVS->setCU_Light2Pos( V ) ;
   light2->Get_Direction( V );
   g_pToonimVS->setCU_Light2Dir( V ) ;

   // initialisation des parametres constants du PS de toon shading
   g_pToonimPS->setCU_EdgeTex( tex.getGLTexName() ) ;
   g_pToonimPS->setCU_DiffuseTex( tex2.getGLTexName() ) ;
   
   g_pToonimPS->setCU_Seuil( seuil ) ;
   light1->Get_Couleur( V );
   g_pToonimPS->setCU_Couleur1( V ) ;
   light2->Get_Couleur( V );
   g_pToonimPS->setCU_Couleur2( V ) ;
   

   // initialisation des parametres d'entree du toon shading
   g_pToonimVS->setU_EyePosition( window.camera() ) ;
   
   // initialisation des parametres de la gestion des traits
   g_pToondeplVS->setCU_SeuilTrait( seuilTrait );
   g_pToondeplVS->setCU_LongCarac ( LCarac );
   g_pToondeplVS->setCU_Vitesse( vitesse );
   
   g_pToondeplPS->setCU_TraitTex( tex3.getGLTexName() );
   
   g_pToondeplVS->setU_EyePosition( window.camera() ) ;
   
   // creation de l'effet associe
   g_pEffect = new CCGEffect ;
   g_pEffect->pushPass( g_pToondeplVS, g_pToondeplPS , true , true, false, false ) ;
   g_pEffect->pushPass( g_pToonimVS, g_pToonimPS , true , true, false, false ) ;
   //g_pEffect->pushPass( g_pToondeplVS  , g_pToondeplPS, false, true, false, true , 0, 0, GL_SRC_COLOR, GL_DST_COLOR ) ;

   // on donne l'effet a utiliser a la fenetre
   window.setEffect( g_pEffect ) ;

   // parametrage de la fenetre
   window.setBackgroundColor( 0.5, 0.5, 0.5 ) ;
   
   
   
   // ---------------------------------------------------

   return application.exec() ;
}
